window.Gleam.initSearch([{"doc":"beencode","title":"beencode","content":"# 🐝 Beencode\n\nA fast Gleam [Bencode](https://en.wikipedia.org/wiki/Bencode) library.\n\nTo add this to your project you can:\n\n```shell\ngleam add beencode@1\n```\n\n```gleam\nimport beencode.{BInt, BList, BString}\n\npub fn main() {\n  let assert Ok(BList([BInt(1), BString(&quot;wibble&quot;)])) =\n    beencode.decode(&lt;&lt;&quot;li1e6:wibblee&quot;:utf8&gt;&gt;)\n\n  let assert &lt;&lt;&quot;li1e6:wibblee&quot;:utf8&gt;&gt; =\n    beencode.encode(BList([BInt(1), BString(&quot;wibble&quot;)]))\n}\n```\n","url":"index.html"},{"doc":"beencode","title":"BDecodeError","content":"pub type BDecodeError {\n  ExpectingEof\n  UnexpectedEof\n  UnexpectedChar(byte_index: Int)\n  EmptyNumber(byte_index: Int)\n  NegativeZero(byte_index: Int)\n  LeadingZero(byte_index: Int)\n  InvalidDictKey(byte_index: Int)\n  StringShorterThanExpected(byte_index: Int)\n}\n An error that might take place while trying to decode a bencoded\n value with the [`decode`](#decode) function.\n\nExpectingEof\n This happens if after decoding a `BValue` we haven&#39;t consumed the entire\n `BitArray` and we still have some other leftover bytes.\n\n txt\n i123ei1e\n ┬───╴┬──\n │    ╰─ After consuming the first integer we&#39;re\n │       left with this piece here but the only way to have\n │       multiple values would be to wrap everything\n │       into a list. This is an invalid bencoded string!\n │\n ╰─ This is consumed just fine.\n \n\n\nUnexpectedEof\n This happens if we unexpectedly reach the end of the `BitArray` while in\n the middle of parsing a value.\n\n\nUnexpectedChar(byte_index: Int)\n\n\nEmptyNumber(byte_index: Int)\n This happens if we run into the invalid empty number `ie`. A number should\n have at least a digit between its start `i` and its end `e`.\n\n\nNegativeZero(byte_index: Int)\n This happens if we find `i-0e` which is not allowed by the bencoding\n specification.\n Zero should only be encoded as `i0e`.\n\n\nLeadingZero(byte_index: Int)\n This happens if a bencoded integer starts with one or more leading zeros,\n which is not allowed by the bencoding specification.\n\n txt\n i001e\n  ┬─\n  ╰─ The leading zeros are not allowed,\n     this should just be `i1e`.\n \n\n\nInvalidDictKey(byte_index: Int)\n This happens if a key of a dictionary is anything else besides a bencoded\n string.\n\n txt\n di1e2:aae\n  ┬─╴\n  ╰─ This key is an integer, not a string.\n \n\n\nStringShorterThanExpected(byte_index: Int)\n This happens if the specified length of a bencoded string is longer than\n the actual string following the `:`.\n\n txt\n 10:aa\n ┬─\n ╰─ According to this the string should be 10-bytes\n    long, but there&#39;s only two bytes here!\n \n\nSynonyms:\nbeencode.BDecodeError\nbeencode BDecodeError","url":"beencode.html#BDecodeError"},{"doc":"beencode","title":"BValue","content":"pub type BValue {\n  BString(BitArray)\n  BInt(Int)\n  BList(List(BValue))\n  BDict(Dict(BitArray, BValue))\n}\n A [bencoded](https://en.wikipedia.org/wiki/Bencode) value.\n\nBString(BitArray)\n The bencoding specification doesn&#39;t enforce strings to be utf8-encoded.\n This means a bencoded string is represented as a Gleam&#39;s `BitArray` and\n not a `String` that would have to be utf8-encoded.\n\n\nBInt(Int)\n\n\nBList(List(BValue))\n\n\nBDict(Dict(BitArray, BValue))\n\n\nSynonyms:\nbeencode.BValue\nbeencode BValue","url":"beencode.html#BValue"},{"doc":"beencode","title":"decode","content":"pub fn decode(input: BitArray) -&gt; Result(BValue, BDecodeError)\n Decodes a [bencoded](https://en.wikipedia.org/wiki/Bencode) `BitArray` into\n a `BValue`.\n\n &gt; ⚠️ According to the bencode specification the keys of a dictionary should\n &gt; always be sorted lexicographically. This decoder is a bit more permissive\n &gt; and will successfully decode a dictionary even if its keys appear in a\n &gt; different order.\n\n ## Examples\n\n \n decode(&lt;&lt;&quot;i1e&quot;:utf8&gt;&gt;)\n // -&gt; BInt(1)\n\n decode(&lt;&lt;&quot;6:wibble&quot;:utf8&gt;&gt;)\n // -&gt; BString(&quot;wibble&quot;)\n\n decode(&lt;&lt;&quot;li1e6:wibblee&quot;:utf8&gt;&gt;)\n // -&gt; BList([BInt(1), BString(&quot;wibble&quot;)])\n\n decode(&lt;&lt;&quot;d6:wibblei1ee&quot;&gt;&gt;)\n // -&gt; BDict(dict.from_list([#(BString(&quot;wibble&quot;), BInt(1))]))\n \n\nSynonyms:\nbeencode.decode\nbeencode decode","url":"beencode.html#decode"},{"doc":"beencode","title":"encode","content":"pub fn encode(value: BValue) -&gt; BitArray\n Encodes a `BValue` into a [bencoded](https://en.wikipedia.org/wiki/Bencode)\n `BitArray`.\n\n ## Examples\n\n \n encode(BInt(1))\n // -&gt; &lt;&lt;&quot;i1e&quot;:utf8&gt;&gt;\n\n encode(BString(&quot;wibble&quot;))\n // -&gt; &lt;&lt;&quot;6:wibble&quot;:utf8&gt;&gt;\n\n encode(BList([BInt(1), BString(&quot;wibble&quot;)]))\n // -&gt; &lt;&lt;&quot;li1e6:wibblee&quot;:utf8&gt;&gt;\n\n encode(BDict(dict.from_list([#(BString(&quot;wibble&quot;), BInt(1))])))\n // -&gt; &lt;&lt;&quot;d6:wibblei1ee&quot;&gt;&gt;\n \n\nSynonyms:\nbeencode.encode\nbeencode encode","url":"beencode.html#encode"},{"doc":"beencode","title":"beencode","content":"","url":"beencode.html"}]);